@@ -1,10 +1,12 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 from decimal import Decimal
 import datetime
 import operator
 from itertools import izip, groupby
-from sql import Column, Literal
+from functools import wraps
+
+from sql import Column, Literal, Null
 from sql.aggregate import Sum
 from sql.conditionals import Coalesce
 
@@ -30,6 +32,14 @@ __all__ = ['TypeTemplate', 'Type', 'OpenType', 'AccountTemplate', 'Account',
     'OpenAgedBalanceStart', 'OpenAgedBalance', 'AgedBalance']
 
 
+def inactive_records(func):
+    @wraps(func)
+    def wrapper(*args, **kwargs):
+        with Transaction().set_context(active_test=False):
+            return func(*args, **kwargs)
+    return wrapper
+
+
 class TypeTemplate(ModelSQL, ModelView):
     'Account Type Template'
     __name__ = 'account.account.type.template'
@@ -70,7 +80,7 @@ class TypeTemplate(ModelSQL, ModelView):
     @staticmethod
     def order_sequence(tables):
         table, _ = tables[None]
-        return [table.sequence == None, table.sequence]
+        return [table.sequence == Null, table.sequence]
 
     @staticmethod
     def default_balance_sheet():
@@ -213,7 +223,7 @@ class Type(ModelSQL, ModelView):
     @staticmethod
     def order_sequence(tables):
         table, _ = tables[None]
-        return [table.sequence == None, table.sequence]
+        return [table.sequence == Null, table.sequence]
 
     @staticmethod
     def default_balance_sheet():
@@ -377,6 +387,12 @@ class AccountTemplate(ModelSQL, ModelView):
             'invisible': Eval('kind') == 'view',
             },
         depends=['kind'])
+    general_ledger_balance = fields.Boolean('General Ledger Balance',
+        states={
+            'invisible': Eval('kind') == 'view',
+            },
+        depends=['kind'],
+        help="Display only the balance in the general ledger report")
 
     @classmethod
     def __setup__(cls):
@@ -401,6 +417,14 @@ class AccountTemplate(ModelSQL, ModelView):
     def default_deferral():
         return True
 
+    @staticmethod
+    def default_party_required():
+        return False
+
+    @staticmethod
+    def default_general_ledger_balance():
+        return False
+
     def get_rec_name(self, name):
         if self.code:
             return self.code + ' - ' + self.name
@@ -409,7 +433,11 @@ class AccountTemplate(ModelSQL, ModelView):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        return ['OR',
+        if clause[1].startswith('!') or clause[1].startswith('not '):
+            bool_op = 'AND'
+        else:
+            bool_op = 'OR'
+        return [bool_op,
             ('code',) + tuple(clause[1:]),
             (cls._rec_name,) + tuple(clause[1:]),
             ]
@@ -431,6 +459,10 @@ class AccountTemplate(ModelSQL, ModelView):
             res['deferral'] = self.deferral
         if not account or account.party_required != self.party_required:
             res['party_required'] = self.party_required
+        if (not account
+                or account.general_ledger_balance !=
+                self.general_ledger_balance):
+            res['general_ledger_balance'] = self.general_ledger_balance
         if not account or account.template != self:
             res['template'] = self.id
         return res
@@ -594,6 +626,12 @@ class Account(ModelSQL, ModelView):
             'invisible': Eval('kind') == 'view',
             },
         depends=['kind'])
+    general_ledger_balance = fields.Boolean('General Ledger Balance',
+        states={
+            'invisible': Eval('kind') == 'view',
+            },
+        depends=['kind'],
+        help="Display only the balance in the general ledger report")
     template = fields.Many2One('account.account.template', 'Template')
 
     @classmethod
@@ -639,6 +677,14 @@ class Account(ModelSQL, ModelView):
     def default_deferral():
         return True
 
+    @staticmethod
+    def default_party_required():
+        return False
+
+    @staticmethod
+    def default_general_ledger_balance():
+        return False
+
     @staticmethod
     def default_kind():
         return 'view'
@@ -809,7 +855,11 @@ class Account(ModelSQL, ModelView):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        return ['OR',
+        if clause[1].startswith('!') or clause[1].startswith('not '):
+            bool_op = 'AND'
+        else:
+            bool_op = 'OR'
+        return [bool_op,
             ('code',) + tuple(clause[1:]),
             (cls._rec_name,) + tuple(clause[1:]),
             ]
@@ -883,7 +933,7 @@ class Account(ModelSQL, ModelView):
             current_type = self.type.id if self.type else None
             template_type = (template2type.get(self.template.type.id)
                 if self.template.type else None)
-            if current_type != template2type:
+            if current_type != template_type:
                 vals['type'] = template_type
             if vals:
                 self.write([self], vals)
@@ -990,7 +1040,11 @@ class AccountDeferral(ModelSQL, ModelView):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        return ['OR',
+        if clause[1].startswith('!') or clause[1].startswith('not '):
+            bool_op = 'AND'
+        else:
+            bool_op = 'OR'
+        return [bool_op,
             ('account.rec_name',) + tuple(clause[1:]),
             ('fiscalyear.rec_name',) + tuple(clause[1:]),
             ]
@@ -1079,10 +1133,8 @@ class PrintGeneralLedgerStart(ModelView):
 
     @fields.depends('fiscalyear')
     def on_change_fiscalyear(self):
-        return {
-            'start_period': None,
-            'end_period': None,
-            }
+        self.start_period = None
+        self.end_period = None
 
 
 class PrintGeneralLedger(Wizard):
@@ -1122,7 +1174,9 @@ class GeneralLedger(Report):
     __name__ = 'account.general_ledger'
 
     @classmethod
-    def parse(cls, report, objects, data, localcontext):
+    def get_context(cls, records, data):
+        report_context = super(GeneralLedger, cls).get_context(records, data)
+
         pool = Pool()
         Account = pool.get('account.account')
         Period = pool.get('account.period')
@@ -1143,7 +1197,9 @@ class GeneralLedger(Report):
                     ('fiscalyear', '=', data['fiscalyear']),
                     ('end_date', '<=', start_period.start_date),
                     ])
-            start_period_ids = [p.id for p in start_periods]
+            start_period_ids += [p.id for p in start_periods]
+        else:
+            start_period = None
 
         with Transaction().set_context(
                 fiscalyear=data['fiscalyear'],
@@ -1167,6 +1223,7 @@ class GeneralLedger(Report):
             end_periods = Period.search([
                     ('fiscalyear', '=', data['fiscalyear']),
                     ])
+            end_period = None
         end_period_ids = [p.id for p in end_periods]
 
         with Transaction().set_context(
@@ -1178,31 +1235,28 @@ class GeneralLedger(Report):
         for account in end_accounts:
             id2end_account[account.id] = account
 
-        periods = end_periods
-        periods.sort(lambda x, y: cmp(x.start_date, y.start_date))
-        localcontext['start_period'] = periods[0]
-        periods.sort(lambda x, y: cmp(x.end_date, y.end_date))
-        localcontext['end_period'] = periods[-1]
-
         if not data['empty_account']:
             account2lines = dict(cls.get_lines(accounts,
                     end_periods, data['posted']))
             accounts = Account.browse([a.id for a in accounts
                     if a in account2lines])
 
-        account_id2lines = cls.lines(accounts,
+        account_id2lines = cls.lines(
+            [a for a in accounts if not a.general_ledger_balance],
             list(set(end_periods).difference(set(start_periods))),
             data['posted'])
 
-        localcontext['accounts'] = accounts
-        localcontext['id2start_account'] = id2start_account
-        localcontext['id2end_account'] = id2end_account
-        localcontext['digits'] = company.currency.digits
-        localcontext['lines'] = lambda account_id: account_id2lines[account_id]
-        localcontext['company'] = company
+        report_context['accounts'] = accounts
+        report_context['id2start_account'] = id2start_account
+        report_context['id2end_account'] = id2end_account
+        report_context['digits'] = company.currency.digits
+        report_context['lines'] = \
+            lambda account_id: account_id2lines[account_id]
+        report_context['company'] = company
+        report_context['start_period'] = start_period
+        report_context['end_period'] = end_period
 
-        return super(GeneralLedger, cls).parse(report, objects, data,
-            localcontext)
+        return report_context
 
     @classmethod
     def get_lines(cls, accounts, periods, posted):
@@ -1216,12 +1270,10 @@ class GeneralLedger(Report):
             clause.append(('move.state', '=', 'posted'))
         lines = MoveLine.search(clause,
             order=[
-                ('account', 'ASC'),  # TODO replace by account.id
+                ('account.id', 'ASC'),
                 ('date', 'ASC'),
                 ])
-        key = operator.attrgetter('account')
-        lines.sort(key=key)
-        return groupby(lines, key)
+        return groupby(lines, operator.attrgetter('account'))
 
     @classmethod
     def lines(cls, accounts, periods, posted):
@@ -1295,10 +1347,8 @@ class PrintTrialBalanceStart(ModelView):
 
     @fields.depends('fiscalyear')
     def on_change_fiscalyear(self):
-        return {
-            'start_period': None,
-            'end_period': None,
-            }
+        self.start_period = None
+        self.end_period = None
 
 
 class PrintTrialBalance(Wizard):
@@ -1338,7 +1388,9 @@ class TrialBalance(Report):
     __name__ = 'account.trial_balance'
 
     @classmethod
-    def parse(cls, report, objects, data, localcontext):
+    def get_context(cls, records, data):
+        report_context = super(TrialBalance, cls).get_context(records, data)
+
         pool = Pool()
         Account = pool.get('account.account')
         Period = pool.get('account.period')
@@ -1420,17 +1472,17 @@ class TrialBalance(Report):
 
         periods = end_periods
 
-        localcontext['accounts'] = accounts
+        report_context['accounts'] = accounts
         periods.sort(key=operator.attrgetter('start_date'))
-        localcontext['start_period'] = periods[0]
+        report_context['start_period'] = periods[0]
         periods.sort(key=operator.attrgetter('end_date'))
-        localcontext['end_period'] = periods[-1]
-        localcontext['company'] = company
-        localcontext['digits'] = company.currency.digits
-        localcontext['sum'] = lambda accounts, field: cls.sum(accounts, field)
+        report_context['end_period'] = periods[-1]
+        report_context['company'] = company
+        report_context['digits'] = company.currency.digits
+        report_context['sum'] = \
+            lambda accounts, field: cls.sum(accounts, field)
 
-        return super(TrialBalance, cls).parse(report, objects, data,
-            localcontext)
+        return report_context
 
     @classmethod
     def sum(cls, accounts, field):
@@ -1533,10 +1585,8 @@ class OpenIncomeStatementStart(ModelView):
 
     @fields.depends('fiscalyear')
     def on_change_fiscalyear(self):
-        return {
-            'start_period': None,
-            'end_period': None,
-            }
+        self.start_period = None
+        self.end_period = None
 
 
 class OpenIncomeStatement(Wizard):
@@ -1628,7 +1678,7 @@ class CreateChart(Wizard):
     start = StateView('account.create_chart.start',
         'account.create_chart_start_view_form', [
             Button('Cancel', 'end', 'tryton-cancel'),
-            Button('Ok', 'account', 'tryton-ok', default=True),
+            Button('OK', 'account', 'tryton-ok', default=True),
             ])
     account = StateView('account.create_chart.account',
         'account.create_chart_account_view_form', [
@@ -1782,9 +1832,10 @@ class UpdateChart(Wizard):
     update = StateTransition()
     succeed = StateView('account.update_chart.succeed',
         'account.update_chart_succeed_view_form', [
-            Button('Ok', 'end', 'tryton-ok', default=True),
+            Button('OK', 'end', 'tryton-ok', default=True),
             ])
 
+    @inactive_records
     def transition_update(self):
         pool = Pool()
         TaxCode = pool.get('account.tax.code')
@@ -1943,7 +1994,10 @@ class ThirdPartyBalance(Report):
     __name__ = 'account.third_party_balance'
 
     @classmethod
-    def parse(cls, report, objects, data, localcontext):
+    def get_context(cls, records, data):
+        report_context = super(ThirdPartyBalance, cls).get_context(records,
+            data)
+
         pool = Pool()
         Party = pool.get('party.party')
         MoveLine = pool.get('account.move.line')
@@ -1958,10 +2012,10 @@ class ThirdPartyBalance(Report):
         account = Account.__table__()
 
         company = Company(data['company'])
-        localcontext['company'] = company
-        localcontext['digits'] = company.currency.digits
-        localcontext['fiscalyear'] = data['fiscalyear']
-        with Transaction().set_context(context=localcontext):
+        report_context['company'] = company
+        report_context['digits'] = company.currency.digits
+        report_context['fiscalyear'] = data['fiscalyear']
+        with Transaction().set_context(context=report_context):
             line_query, _ = MoveLine.query_get(line)
         if data['posted']:
             posted_clause = move.state == 'posted'
@@ -1971,12 +2025,12 @@ class ThirdPartyBalance(Report):
         cursor.execute(*line.join(move, condition=line.move == move.id
                 ).join(account, condition=line.account == account.id
                 ).select(line.party, Sum(line.debit), Sum(line.credit),
-                where=(line.party != None)
+                where=(line.party != Null)
                 & account.active
                 & account.kind.in_(('payable', 'receivable'))
                 & (account.company == data['company'])
                 & ((line.maturity_date <= Date.today())
-                    | (line.maturity_date == None))
+                    | (line.maturity_date == Null))
                 & line_query & posted_clause,
                 group_by=line.party,
                 having=(Sum(line.debit) != 0) | (Sum(line.credit) != 0)))
@@ -1992,12 +2046,11 @@ class ThirdPartyBalance(Report):
             'solde': x[1] - x[2],
             } for x in res]
         objects.sort(lambda x, y: cmp(x['name'], y['name']))
-        localcontext['total_debit'] = sum((x['debit'] for x in objects))
-        localcontext['total_credit'] = sum((x['credit'] for x in objects))
-        localcontext['total_solde'] = sum((x['solde'] for x in objects))
+        report_context['total_debit'] = sum((x['debit'] for x in objects))
+        report_context['total_credit'] = sum((x['credit'] for x in objects))
+        report_context['total_solde'] = sum((x['solde'] for x in objects))
 
-        return super(ThirdPartyBalance, cls).parse(report, objects, data,
-            localcontext)
+        return report_context
 
 
 class OpenAgedBalanceStart(ModelView):
@@ -2084,7 +2137,9 @@ class AgedBalance(Report):
     __name__ = 'account.aged_balance'
 
     @classmethod
-    def parse(cls, report, objects, data, localcontext):
+    def get_context(cls, records, data):
+        report_context = super(AgedBalance, cls).get_context(records, data)
+
         pool = Pool()
         Party = pool.get('party.party')
         MoveLine = pool.get('account.move.line')
@@ -2099,9 +2154,9 @@ class AgedBalance(Report):
         account = Account.__table__()
 
         company = Company(data['company'])
-        localcontext['digits'] = company.currency.digits
-        localcontext['posted'] = data['posted']
-        with Transaction().set_context(context=localcontext):
+        report_context['digits'] = company.currency.digits
+        report_context['posted'] = data['posted']
+        with Transaction().set_context(context=report_context):
             line_query, _ = MoveLine.query_get(line)
 
         terms = (data['term1'], data['term2'], data['term3'])
@@ -2126,10 +2181,10 @@ class AgedBalance(Report):
             cursor.execute(*line.join(move, condition=line.move == move.id
                     ).join(account, condition=line.account == account.id
                     ).select(line.party, Sum(line.debit) - Sum(line.credit),
-                    where=(line.party != None)
+                    where=(line.party != Null)
                     & account.active
                     & account.kind.in_(kind)
-                    & (line.reconciliation == None)
+                    & (line.reconciliation == Null)
                     & (account.company == data['company'])
                     & term_query & line_query,
                     group_by=line.party,
@@ -2144,20 +2199,19 @@ class AgedBalance(Report):
             ('id', 'in', [k for k in res.iterkeys()]),
             ])
 
-        localcontext['main_title'] = data['balance_type']
-        localcontext['unit'] = data['unit']
+        report_context['main_title'] = data['balance_type']
+        report_context['unit'] = data['unit']
         for i in range(3):
-            localcontext['total' + str(i)] = sum(
+            report_context['total' + str(i)] = sum(
                 (v[i] for v in res.itervalues()))
-            localcontext['term' + str(i)] = terms[i]
+            report_context['term' + str(i)] = terms[i]
 
-        localcontext['company'] = company
-        localcontext['parties'] = [{
+        report_context['company'] = company
+        report_context['parties'] = [{
                 'name': p.rec_name,
                 'amount0': res[p.id][0],
                 'amount1': res[p.id][1],
                 'amount2': res[p.id][2],
                 } for p in parties]
 
-        return super(AgedBalance, cls).parse(report, objects, data,
-            localcontext)
+        return report_context
