@@ -1,5 +1,7 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
+from sql import Null
+
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.wizard import Wizard, StateTransition
 from trytond import backend
@@ -78,7 +80,7 @@ class JournalViewColumn(ModelSQL, ModelView):
     @staticmethod
     def order_sequence(tables):
         table, _ = tables[None]
-        return [table.sequence == None, table.sequence]
+        return [table.sequence == Null, table.sequence]
 
     @staticmethod
     def default_required():
@@ -97,7 +99,7 @@ class Journal(ModelSQL, ModelView):
     active = fields.Boolean('Active', select=True)
     type = fields.Selection('get_types', 'Type', required=True)
     view = fields.Many2One('account.journal.view', 'View')
-    update_posted = fields.Boolean('Allow cancelling moves')
+    update_posted = fields.Boolean('Allow updating posted moves')
     sequence = fields.Property(fields.Many2One('ir.sequence', 'Sequence',
             domain=[('code', '=', 'account.journal')],
             context={'code': 'account.journal'},
@@ -140,8 +142,8 @@ class Journal(ModelSQL, ModelView):
         # Migration from 1.0 sequence Many2One change into Property
         if table.column_exist('sequence'):
             Property = Pool().get('ir.property')
-            table = cls.__table__()
-            cursor.execute(*table.select(table.id, table.sequence))
+            sql_table = cls.__table__()
+            cursor.execute(*sql_table.select(sql_table.id, sql_table.sequence))
             for journal_id, sequence_id in cursor.fetchall():
                 Property.set('sequence', cls._name,
                         journal_id, (sequence_id and
@@ -168,7 +170,11 @@ class Journal(ModelSQL, ModelView):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        return ['OR',
+        if clause[1].startswith('!') or clause[1].startswith('not '):
+            bool_op = 'AND'
+        else:
+            bool_op = 'OR'
+        return [bool_op,
             ('code',) + tuple(clause[1:]),
             (cls._rec_name,) + tuple(clause[1:]),
             ]
